#!/usr/local/bin/perl -w

########################################################
# 
# ncDNAlign.2.getGwAln.pl
#    part of the NcDNAlign alignment pipeline, step (2)
#
# Authors: Dominic Rose, Jana Hertel
#
# Contact: dominic@bioinf.uni-leipzig.de
#          jana@bioinf.uni-leipzig.de
# http://www.bioinf.uni-leipzig.de/Software/NcDNAlign/
#
########################################################

use NcDNAlign;
use strict;
use Getopt::Long;
use Pod::Usage;

my ($opt_conf, $opt_help, $opt_version, $opt_man, $opt_s) = ('', '', '', '', 0);

GetOptions(
	"c|conf=s" 	=> \$opt_conf,
	"h|help"	=> \$opt_help,
	"v|version"	=> \$opt_version,
	"man"		=> \$opt_man,
	"s|silent"	=> \$opt_s
) or pod2usage(2);

pod2usage(1) if ($opt_help || $opt_s !~ /^[1|0]$/);
pod2usage(-exitstatus => 0, -verbose => 2) if $opt_man;

if($opt_version) {
	print "This is ncDNAlign.2.getGwAln.pl, part of the NcDNAlign package, version: $VERSION.\n$url\n";
	exit;
}

if(!$opt_conf) {
	pod2usage(1);
}

# ------ Start ----------
loadConfig($opt_conf);

if( !(-d "$C{p_dir}/blast") ) {
	my $cmd = "mkdir $C{p_dir}/blast";
	print "$cmd\n" if(!$opt_s);
	system("$cmd"); 
}

# loop through the organisms and set up formatdb files
foreach my $k (sort keys %ORG) {
	#print "$k $ORG{$k}{ncregions}\n";
	# check for formatdb files
	if( !(	-e "$C{source_dir}/$ORG{$k}{ncregions}.nhr" &&
		-e "$C{source_dir}/$ORG{$k}{ncregions}.nin" &&
		-e "$C{source_dir}/$ORG{$k}{ncregions}.nsd" &&
		-e "$C{source_dir}/$ORG{$k}{ncregions}.nsi" &&
		-e "$C{source_dir}/$ORG{$k}{ncregions}.nsq") 
	) {
		
		if(-f "$C{source_dir}/$ORG{$k}{ncregions}") {
			
			print "Running formatdb:\n" if(!$opt_s);
			my $cmd = "formatdb -i $C{source_dir}/$ORG{$k}{ncregions} -p F -o";
			print "$cmd\n" if(!$opt_s);
			system($cmd) == 0 or die $die_msg."system call '$cmd' failed: $?"; # create blast DB for the first subject organism
		}
	}
}

# perform blast search
my $i=2;
my $cmd = '';

print "The following command lines will be executed:\n" if(!$opt_s);

foreach my $name (sort keys %ORG) {
	
	if($C{target_name} ne $name) { # who is subject? everything exept the target
		$cmd = "$C{blast_cmd} -d $C{source_dir}/$ORG{$name}{ncregions} -i $C{source_dir}/$ORG{$C{target_name}}{ncregions} -m 8 -e $C{cutoff_evalue} -o $C{p_dir}/blast/$i.out";
		
		$i++;
		if(!$opt_s) {
			print "----- $C{target_name} vs $name -----\n";
			print "\t $cmd\n";
		}
	}
}

$i=2;
$cmd = '';

print "\nComputation now begins, you may stop here and run/distribute the commands above by yourself to speed up overall computation time!\n" if(!$opt_s);

foreach my $name (sort keys %ORG) {
	
	if($C{target_name} ne $name) { # who is subject? everything exept the target
		$cmd = "$C{blast_cmd} -d $C{source_dir}/$ORG{$name}{ncregions} -i $C{source_dir}/$ORG{$C{target_name}}{ncregions} -m 8 -e $C{cutoff_evalue} -o $C{p_dir}/blast/$i.out";
		
		$i++;
		if(!$opt_s) {
			print "----- $C{target_name} vs $name -----\n" ;
			print "$cmd\n";
		}
		system ($cmd) == 0 or die $die_msg."system call '$cmd' failed: $?";		
	}
}


__END__

=head1 NAME

C<ncDNAlign.2.getGwAln.pl> - part of the NcDNAlign alignment pipeline, step (2)

Wraps a pairwise BLAST search of the target genome against all other organisms listed in the configurations file. 
The abbreviation GwAln stads for "genomewide alignments".

=head1 SYNOPSIS

ncDNAlign.2.getGwAln.pl [options]

Type 

    $ ncDNAlign.2.getGwAln.pl --help

or

   $  ncDNAlign.2.getGwAln.pl --man 

for a detailed man page.


=head1 OPTIONS

=over 8

=item B<-c, --conf> FILE

Path to central NcDNAlign configuration file

=item B<-s, --silent> [0|1]

Silent mode, avoid printing to STDOUT
0=OFF (Default), 1=ON

=item B<-v, --version>

Prints version information and exits.

=item B<-h, --help>

Prints a short help message and exits.

=item B<--man>

Prints a detailed manual page and exits.

=back

=head1 DESCRIPTION

The script performs a pairwise nucleotide BLAST search using BLASTN to align the reference sequence heuristically against all other species. The best hit for each query subsequence is retained if the EVALUE of the BLAST search does not exceed the EVALUE-threshold and the query length is larger than a certain minimal length. Both thresholds are given in the config file. Results of the BLAST search are stored at the directory 'blast'.

=head1 EXAMPLES

$ ncDNAlign.2.getGwAln.pl -c config-file.cfg

=head1 AUTHORS

Dominic Rose (dominic@bioinf.uni-leipzig.de)

=head1 AVAILABILITY

http://www.bioinf.uni-leipzig.de/Software/NcDNAlign/

=cut
